@@ -21,7 +21,6 @@ import re
 import collections
 import io
 import itertools
-import warnings
 import six
 from six.moves import urllib
 # Make defaultdict compatible with versions of python older than 2.4
@@ -31,12 +30,18 @@ except AttributeError:
     import _utils
     collections.defaultdict = _utils.defaultdict
 
-from Bio.Seq import UnknownSeq
+unknown_seq_avail = False
+try:
+    from Bio.Seq import UnknownSeq
+    unknown_seq_avail = True
+except ImportError:
+    # Starting with biopython 1.81, has been removed
+    from Bio.Seq import _UndefinedSequenceData
+    from Bio.Seq import Seq
+
 from Bio.SeqRecord import SeqRecord
 from Bio import SeqFeature
 from Bio import SeqIO
-from Bio import BiopythonDeprecationWarning
-warnings.simplefilter("ignore", BiopythonDeprecationWarning)
 
 def _gff_line_map(line, params):
     """Map part of Map-Reduce; parses a line of GFF into a dictionary.
@@ -69,7 +74,7 @@ def _gff_line_map(line, params):
 
         GFF3 has key value pairs like:
           count=9;gene=amx-2;sequence=SAGE:aacggagccg
-        GFF2 and GTF have:           
+        GFF2 and GTF have:
           Sequence "Y74C9A" ; Note "Clone Y74C9A; Genbank AC024206"
           name "fgenesh1_pg.C_chr_1000003"; transcriptId 869
         """
@@ -170,7 +175,7 @@ def _gff_line_map(line, params):
         should_do = True
         if params.limit_info:
             for limit_name, limit_values in params.limit_info.items():
-                cur_id = tuple([parts[i] for i in 
+                cur_id = tuple([parts[i] for i in
                     params.filter_info[limit_name]])
                 if cur_id not in limit_values:
                     should_do = False
@@ -286,7 +291,7 @@ class _AbstractMapReduceGFF:
     information.
     """
     def __init__(self, create_missing=True):
-        """Initialize GFF parser 
+        """Initialize GFF parser
 
         create_missing - If True, create blank records for GFF ids not in
         the base_dict. If False, an error will be raised.
@@ -305,7 +310,7 @@ class _AbstractMapReduceGFF:
         limit_info - A dictionary specifying the regions of the GFF file
         which should be extracted. This allows only relevant portions of a file
         to be parsed.
-        
+
         base_dict - A base dictionary of SeqRecord objects which may be
         pre-populated with sequences and other features. The new features from
         the GFF file will be added to this dictionary.
@@ -460,7 +465,7 @@ class _AbstractMapReduceGFF:
             # one child, do not nest it
             if len(cur_children) == 1:
                 rec_id, child = cur_children[0]
-                loc = (child.location.nofuzzy_start, child.location.nofuzzy_end)
+                loc = (child.location.start, child.location.end)
                 rec, base = self._get_rec(base,
                                           dict(rec_id=rec_id, location=loc))
                 rec.features.append(child)
@@ -536,11 +541,16 @@ class _AbstractMapReduceGFF:
         if match_id:
             cur_rec = base[match_id]
             # update generated unknown sequences with the expected maximum length
-            if isinstance(cur_rec.seq, UnknownSeq):
+            if unknown_seq_avail and isinstance(cur_rec.seq, UnknownSeq):
                 cur_rec.seq._length = max([max_loc, cur_rec.seq._length])
+            elif not unknown_seq_avail and isinstance(cur_rec.seq._data, _UndefinedSequenceData):
+                cur_rec.seq._data._length = max([max_loc, cur_rec.seq._data._length])
             return cur_rec, base
         elif self._create_missing:
-            new_rec = SeqRecord(UnknownSeq(max_loc), info_dict['rec_id'])
+            if unknown_seq_avail:
+                new_rec = SeqRecord(UnknownSeq(max_loc), info_dict['rec_id'])
+            else:
+                new_rec = SeqRecord(Seq(None, length=max_loc), info_dict['rec_id'])
             base[info_dict['rec_id']] = new_rec
             return new_rec, base
         else:
@@ -551,13 +561,13 @@ class _AbstractMapReduceGFF:
         """Add a new feature that is missing from the GFF file.
         """
         base_rec_id = list(set(c[0] for c in cur_children))
-        child_strands = list(set(c[1].strand for c in cur_children))
+        child_strands = list(set(c[1].location.strand for c in cur_children))
         inferred_strand = child_strands[0] if len(child_strands) == 1 else None
         assert len(base_rec_id) > 0
         feature_dict = dict(id=parent_id, strand=inferred_strand,
                             type="inferred_parent", quals=dict(ID=[parent_id]),
                             rec_id=base_rec_id[0])
-        coords = [(c.location.nofuzzy_start, c.location.nofuzzy_end)
+        coords = [(c.location.start, c.location.end)
                   for r, c in cur_children]
         feature_dict["location"] = (min([c[0] for c in coords]),
                                     max([c[1] for c in coords]))
@@ -574,9 +584,10 @@ class _AbstractMapReduceGFF:
     def _get_feature(self, feature_dict):
         """Retrieve a Biopython feature from our dictionary representation.
         """
-        location = SeqFeature.FeatureLocation(*feature_dict['location'])
-        new_feature = SeqFeature.SeqFeature(location, feature_dict['type'],
-                id=feature_dict['id'], strand=feature_dict['strand'])
+        #location = SeqFeature.FeatureLocation(*feature_dict['location'])
+        rstart, rend = feature_dict['location']
+        new_feature = SeqFeature.SeqFeature(SeqFeature.SimpleLocation(start=rstart, end=rend, strand=feature_dict['strand']), feature_dict['type'],
+                id=feature_dict['id'])
         # Support for Biopython 1.68 and above, which removed sub_features
         if not hasattr(new_feature, "sub_features"):
             new_feature.sub_features = []
@@ -654,7 +665,7 @@ class GFFParser(_AbstractMapReduceGFF):
     def __init__(self, line_adjust_fn=None, create_missing=True):
         _AbstractMapReduceGFF.__init__(self, create_missing=create_missing)
         self._line_adjust_fn = line_adjust_fn
-    
+
     def _gff_process(self, gff_files, limit_info, target_lines):
         """Process GFF addition without any parallelization.
 
@@ -704,7 +715,7 @@ class GFFParser(_AbstractMapReduceGFF):
                 yield out_info.get_results()
                 out_info = _GFFParserLocalOut((target_lines is not None and
                         target_lines > 1))
-            if (results and results[0][0] == 'directive' and 
+            if (results and results[0][0] == 'directive' and
                     results[0][1] == 'FASTA'):
                 found_seqs = True
                 break
@@ -741,7 +752,7 @@ class DiscoGFFParser(_AbstractMapReduceGFF):
     """
     def __init__(self, disco_host, create_missing=True):
         """Initialize parser.
-        
+
         disco_host - Web reference to a Disco host which will be used for
         parallelizing the GFF reading job.
         """
@@ -755,7 +766,7 @@ class DiscoGFFParser(_AbstractMapReduceGFF):
         # make these imports local; only need them when using disco
         import simplejson
         import disco
-        # absolute path names unless they are special disco files 
+        # absolute path names unless they are special disco files
         full_files = []
         for f in gff_files:
             if f.split(":")[0] != "disco":
@@ -829,7 +840,7 @@ class GFFExaminer:
     def __init__(self):
         self._filter_info = dict(gff_id = [0], gff_source_type = [1, 2],
                 gff_source = [1], gff_type = [2])
-    
+
     def _get_local_params(self, limit_info=None):
         class _LocalParams:
             def __init__(self):
@@ -838,13 +849,13 @@ class GFFExaminer:
         params.limit_info = limit_info
         params.filter_info = self._filter_info
         return params
-    
+
     @_file_or_handle
     def available_limits(self, gff_handle):
         """Return dictionary information on possible limits for this file.
 
         This returns a nested dictionary with the following structure:
-        
+
         keys -- names of items to filter by
         values -- dictionary with:
             keys -- filter choice
@@ -884,7 +895,7 @@ class GFFExaminer:
 
         keys -- tuple of (source, type) for each parent
         values -- tuple of (source, type) as children of that parent
-        
+
         Not a parallelized map-reduce implementation.
         """
         # collect all of the parent and child types mapped to IDs
